/*********************************************************************
 *
 *                Microchip USB C18 Firmware Version 1.2
 *
 *********************************************************************
 * FileName:        user.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 3.11+
 * Company:         Microchip Technology, Inc.
 *
 ********************************************************************/

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include "system\typedefs.h"
#include "system\usb\usb.h"
#include "io_cfg.h"             // I/O pin mapping
#include "user\user.h"
#include <delays.h>
#include <adc.h>
#include <stdlib.h>
#include <math.h>

/** V A R I A B L E S ********************************************************/
#pragma udata
float Freq;
int i, j;
unsigned char mask;	
unsigned int Result;
float Para;
union{
	unsigned long	Phase;			// gݒl
	unsigned char	iPhase[4];
} DDSdata;
float logData0,logData1;			//dBl
char input_buffer[48];				// USB̓obt@
char output_buffer[48];				// USBo̓obt@
char MsgTX[] = "CH0=+xx.x  CH1=+xx.x\r\n";


/** P R I V A T E  P R O T O T Y P E S ***************************************/
void BlinkUSBStatus(void);
BOOL Switch2IsPressed(void);
void itostring(char digit, unsigned int data, char *buffer);
void serial_out(float data);
void ftostring(int seisu, int shousu, float data, char *buffer);
	
/** D E C L A R A T I O N S **************************************************/
#pragma code
void UserInit(void)
{
	TRISA = 0x03;							//RA0,1̂ݓ
	TRISB = 0;								//So
	TRISC = 0;								//So
	LATA = 0;								//|[gSNA
	LATB = 0;
	LATC = 0;
	Freq = 100000.0;
	// A/DRo[^ݒ
	OpenADC(ADC_FOSC_64 & ADC_RIGHT_JUST & ADC_16_TAD, ADC_CH0
	 					& ADC_INT_OFF & ADC_REF_VDD_VSS, ADC_2ANA);
	
	/*** DDS̐ݒ萧 VA[hɐݒ	***/	
	RST = 1;							//RESETo
	Delay10TCYx(1);
	RST = 0;
	W_CLK =1;							//W0[h
	W_CLK = 0;
	FQ_UD = 1;							//VA[hݒ
	FQ_UD = 0;	
}//end UserInit

/******************************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *
 * Note:            None
 *****************************************************************************/
void ProcessIO(void)
{
//    BlinkUSBStatus();											// USBڑԕ\
 
 	/*** USB ڑmF (ڑȂ璼^[) ***/
	if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;
	/*** R}h͂Ə  ***/
 	if(getsUSBUSART(input_buffer,48)){					// f[^M|[
		mLED_1_Toggle();								// LED]
  		switch(input_buffer[0])	{						// ŏ̂P`FbN
   			case '0':	  								// ݒƃI[v
        	   	if(mUSBUSARTIsTxTrfReady())				// M\H
     			 	putrsUSBUSART("OK");				// OKf[^M
     			break;         			 	
   			case '1':									// ݒ
				Freq = atof(&input_buffer[1]);			// f[^擾
				serial_out(Freq); 
     		 	break;
        	case '2':									// f[^ڑM
		 		SetChanADC(ADC_CH0);					// `l0I
				ConvertADC();							// ϊJn
				while(BusyADC());						// ϊI҂
				Result = ReadADC();						// CH0
				/* dBɕϊ@Vref=4.73V@0dB=2.19VŎl
				   40dB*4.73/1024=0.185 		 */
   				logData0 = 0.219*Result - 87.0;			//dBɕϊiIClj
				if(logData0 < 0){						//H
					MsgTX[4] = '-';						//-LZbg
					logData0 = fabs(logData0);			//Βlɕϊ
				}
				else
					MsgTX[4] = '+';
  				ftostring(2, 1, logData0, MsgTX+5);
        		SetChanADC(ADC_CH1);					// `l1I
				ConvertADC();							// ϊJn
				while(BusyADC());						// ϊI҂
				Result = ReadADC();						// CH1
				logData1 = 0.219*Result - 87.0;			//dBɕϊiIClj
				if(logData1 < 0){						//H
					MsgTX[15] = '-';					//-LZbg
					logData1 = fabs(logData1);			//Βlɕϊ
				}
				else
					MsgTX[15] = '+';
		       	ftostring(2, 1, logData1, MsgTX+16);
      		   	if(mUSBUSARTIsTxTrfReady())				// M\H
     		   	 	mUSBUSARTTxRam((byte*)(MsgTX), 22);	// f[^M
        		break;
	   		default:
        	   	if(mUSBUSARTIsTxTrfReady())				// M\H
     			 	putrsUSBUSART("NG");				// NGf[^M
	   			 	break;
	 	}
	}

}

/***********************************************************
* AD9851@\DDS֐
* @DDSւ̑MtH[}bg͉L
* @ W0  Freq bit0(LSB)
* @ W1  Freq bit1@@@Freq= (ݒl~NbN)232
* @ ------------
* @ W31 Freq bit31(MSB)
* @ W32 x6 REFCLK Multiplier Enable
*  @W33 0
*  @W34 Power Down at 1
*  @W35 Phase bit0(LSB)   360x/32Ŏw
*  @--------------
*  @W39 Phase bit4(MSB)
*  NbN24MHz~6=144MHz
***********************************************************/
void	serial_out(float data){
	/* gݒ萧s ۂ̎g143.9958ŕ␳@*/
	Para = 29.82703 * data;				//232恀144MHz=29.82616
	DDSdata.Phase = (long)Para;			//
	////@gl32rbgo͂
	for(i=0; i<4; i++)	{
		mask = 0x01;					//rbg}XN̏ݒ
		for(j=0; j<8; j++){
			if(DDSdata.iPhase[i] & mask)
				S_IN =1;
			else
				S_IN = 0;
			W_CLK =1;					//rbgMNbN
			W_CLK = 0;
			mask = mask << 1;			//rbg}XÑrbgVtg
		}
	}
	///@ݒ萧8rbgo
	mask = 0x01;						//rbg}XN̏ݒ
	for(j=0; j<8; j++)	{
		if(0x01 & mask)					//U{[ĥ݂P
			S_IN = 1;
		else
			S_IN = 0;
		W_CLK = 1;						//rbgMNbN
		W_CLK = 0;
		mask = mask << 1;				//rbg}XN̂PrbgVtg
	}
	FQ_UD =1;							//gl[hw
	FQ_UD =0;	
	FQ_UD = 0;	
}

/******************************************************************************
 * int l當ɕϊ
 *****************************************************************************/
void itostring(char digit, unsigned int data, char *buffer)
{
	char i;
	buffer += digit;						// ̍Ō
	for(i=digit; i>0; i--) {				// ŉʌʂ
		buffer--;						// |C^[P
		*buffer = (data % 10) + '0';		// ̌l𕶎ɂĊi[
		data = data / 10;					// -1
	}
}

/******************************************************************************
 * Floatl當֕ϊ
 *@vL͂TȉƂ邱
 *****************************************************************************/
void ftostring(int seisu, int shousu, float data, char *buffer)
{
	unsigned int i, dumy;

	if(shousu != 0)							//肩
		buffer += seisu+shousu+1;			//Š{_
	else									//Ȃ̂Ƃ
		buffer += seisu + shousu;			//Š̂
	*buffer = 0;							//zŌ\0i[
	buffer--;								//z|C^-1
	for(i=0; i<shousu; i++)					//𐮐ɕϊ
		data = data * 10;					//PO{
	dumy = (unsigned int) (data + 0.5);		//ľܓĐɕϊ
	for(i=shousu; i>0; i--)	{				//JԂ
		*buffer =(dumy % 10)+'0';			//l𕶎ɕϊi[
		buffer--;							//i[ꏊʂʂ
		dumy = (unsigned int)(dumy / 10);	//̌
	}
	if(shousu != 0) {						//0Ȃ珬_Ȃ
		*buffer = '.';						//_i[
		buffer--;							//|C^-1
	}
	for(i=seisu; i>0; i--) {				//JԂ
		*buffer = (dumy % 10)+'0';			//l𕶎ɕϊi[
		buffer--;							//|C^-1
		dumy = (unsigned int)(dumy / 10);	//̌
	}
}    
/******************************************************************************
 * Function:        void BlinkUSBStatus(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        BlinkUSBStatus turns on and off LEDs corresponding to
 *                  the USB device state.
 *
 * Note:            mLED macros can be found in io_cfg.h
 *                  usb_device_state is declared in usbmmap.c and is modified
 *                  in usbdrv.c, usbctrltrf.c, and usb9.c
 *****************************************************************************/
void BlinkUSBStatus(void)
{
    static word led_count=0;

    if(led_count == 0)led_count = 10000U;
    led_count--;

    #define mLED_Both_Off()         {mLED_1_Off();mLED_2_Off();}
    #define mLED_Both_On()          {mLED_1_On();mLED_2_On();}
    #define mLED_Only_1_On()        {mLED_1_On();mLED_2_Off();}
    #define mLED_Only_2_On()        {mLED_1_Off();mLED_2_On();}

    if(UCONbits.SUSPND == 1)
    {
        if(led_count==0)
        {
            mLED_1_Toggle();
            mLED_2 = mLED_1;        // Both blink at the same time
        }//end if
    }
    else
    {
        if(usb_device_state == DETACHED_STATE)
        {
            mLED_Both_Off();
        }
        else if(usb_device_state == ATTACHED_STATE)
        {
            mLED_Both_On();
        }
        else if(usb_device_state == POWERED_STATE)
        {
            mLED_Only_1_On();
        }
        else if(usb_device_state == DEFAULT_STATE)
        {
            mLED_Only_2_On();
        }
        else if(usb_device_state == ADDRESS_STATE)
        {
            if(led_count == 0)
            {
                mLED_1_Toggle();
                mLED_2_Off();
            }//end if
        }
        else if(usb_device_state == CONFIGURED_STATE)
        {
            if(led_count==0)
            {
                mLED_1_Toggle();
                mLED_2 = !mLED_1;       // Alternate blink
            }//end if
        }//end if(...)
    }//end if(UCONbits.SUSPND...)

}//end BlinkUSBStatus

/** EOF user.c ***************************************************************/
